/*
 * utility.hpp
 *
 * This file is part of Algorithms library toolkit.
 * Copyright (C) 2017 Jan Travnicek (jan.travnicek@fit.cvut.cz)

 * Algorithms library toolkit is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Algorithms library toolkit is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Algorithms library toolkit.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Created on: Apr 7, 2016
 * Author: Jan Travnicek
 */

#ifndef __UTILITY_HPP_
#define __UTILITY_HPP_

#include <utility>

#include "type_traits.hpp"

#include <extensions/container/ptr_value.hpp>

namespace ext {

/**
 * \brief
 * Allow moving of copied instance of the source. Designed for copyconstructible objects.
 *
 * \tparam T the type of processed instance
 *
 * \param param the source instance
 *
 * \return copy to be moved from
 */
template < class T, typename std::enable_if < ! ext::has_clone < T >::value >::type * = nullptr >
inline T move_copy ( const T & param ) {
	return T ( param );
}

/**
 * \brief
 * Allow moving of copied instance of the source. Designed for cloneable objects.
 *
 * \tparam T the type of processed instance
 *
 * \param param the source instance
 *
 * \return wrapper of dynamically allocated copy supporting cast to rvalue reference to T
 */
template < class T, typename std::enable_if < ext::has_clone < T >::value >::type * = nullptr >
inline ptr_value < T > move_copy ( const T & param ) {
	return ptr_value < T > ( param );
}

} /* namespace ext */

#endif /* __UTILITY_HPP_ */
